@@ -520,15 +520,13 @@ prune_node (BonoboUIEngine *engine,
 #endif
 
 			gtk_widget_destroy (item);
+			widget_unref (&info->widget);
 		} else {
 			if (save)
 				custom_widget_unparent (info);
 /*			printf ("leave widget '%s'\n",
 			bonobo_ui_xml_make_path (node));*/
 		}
-
-		if (!save)
-			widget_unref (&info->widget);
 	}
 }
 
@@ -1447,6 +1445,7 @@ real_exec_verb (BonoboUIEngine *engine,
 		const char     *component_name,
 		const char     *verb)
 {
+	char *verb_cpy;
 	Bonobo_UIComponent component;
 
 	g_return_if_fail (verb != NULL);
@@ -1457,13 +1456,17 @@ real_exec_verb (BonoboUIEngine *engine,
 
 	component = sub_component_objref (engine, component_name);
 
+	verb_cpy = g_strdup (verb);
+
 	if (component != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
 
+		CORBA_Object_duplicate (component, &ev);
+
 		Bonobo_UIComponent_execVerb (
-			component, verb, &ev);
+			component, verb_cpy, &ev);
 
 		if (engine->priv->container)
 			bonobo_object_check_env (
@@ -1471,14 +1474,16 @@ real_exec_verb (BonoboUIEngine *engine,
 				component, &ev);
 
 		if (BONOBO_EX (&ev))
-			g_warning ("Exception executing verb '%s' '%s'"
-				   "major %d, %s",
-				   verb, component_name, ev._major,
+			g_warning ("Exception executing verb '%s'"
+				   "major %d, %s", verb_cpy, ev._major,
 				   BONOBO_EX_REPOID (&ev));
+
+		CORBA_Object_release (component, &ev);
 		
 		CORBA_exception_free (&ev);
 	}
 
+	g_free (verb_cpy);
 	g_object_unref (engine);
 }
 
@@ -1724,7 +1729,7 @@ impl_emit_event_on (BonoboUIEngine *engine,
 {
 	const char      *id;
 	BonoboUIXmlData *data;
-	char            *component_id, *real_id;
+	char            *component_id, *real_id, *real_state;
 
 	g_return_if_fail (node != NULL);
 
@@ -1738,18 +1743,20 @@ impl_emit_event_on (BonoboUIEngine *engine,
 
 	component_id = g_strdup (data->id);
 	real_id      = g_strdup (id);
+	real_state   = g_strdup (state);
 
 	/* This could invoke a CORBA method that might de-register the component */
 	set_cmd_attr (engine, node, state_id, state, TRUE);
 
 	real_emit_ui_event (engine, component_id, real_id,
 			    Bonobo_UIComponent_STATE_CHANGED,
-			    state);
+			    real_state);
 
 	g_object_unref (engine);
 
-	g_free (component_id);
+	g_free (real_state);
 	g_free (real_id);
+	g_free (component_id);
 }
 
 void
@@ -2293,6 +2300,8 @@ do_sync (BonoboUIEngine *engine,
 		 G_OBJECT_CLASS_NAME (sync),
 		 bonobo_ui_xml_make_path (node));
 #endif
+	bonobo_ui_node_ref (node);
+
 	if (bonobo_ui_node_parent (node) == engine->priv->tree->root)
 		bonobo_ui_sync_update_root (sync, node);
 
@@ -2315,6 +2324,8 @@ do_sync (BonoboUIEngine *engine,
 	}
 
 	bonobo_ui_xml_clean (engine->priv->tree, node);
+
+	bonobo_ui_node_unref (node);
 }
 
 static void
@@ -2348,6 +2359,9 @@ seek_dirty (BonoboUIEngine *engine,
  * 
  * This function is used to write recursive synchronizers
  * and is intended only for internal / privilaged use.
+ *
+ * By the time this returns, due to re-enterancy, node
+ * points at undefined memory.
  **/
 void
 bonobo_ui_engine_update_node (BonoboUIEngine *engine,
@@ -2976,13 +2990,17 @@ bonobo_ui_engine_stamp_root (BonoboUIEngine *engine,
 			     GtkWidget      *widget)
 {
 	NodeInfo *info;
+	GtkWidget *new_root;
 
 	if (!node)
 		return;
 
 	info = bonobo_ui_xml_get_data (engine->priv->tree, node);
 
-	info->widget = widget ? gtk_widget_ref (widget) : NULL;
+	new_root = widget ? gtk_widget_ref (widget) : NULL;
+	if (info->widget)
+		gtk_widget_unref (info->widget);
+	info->widget = new_root;
 	info->type |= ROOT_WIDGET;
 
 	bonobo_ui_engine_widget_attach_node (widget, node);
